/**
 * @file	xmldoc.h
 * @brief	#XmlDoc class definition.
 * @date	2004.02.14
 * @author	Steel Wheels Project
 * @par Copyright
 *   Copyright (C) 2004 Steel Wheels Project.
 *   All right reserved.
 * @par Description
 *   #XmlDoc class definition.
 *   #XmlDoc class operates entire data for a XML file.
 */

#ifndef XMLDOC_H_INCLUDED
#define XMLDOC_H_INCLUDED

#include <LibXmlKit/xmlnode.h>

/**
 * @brief	Class for XML document.
 * @par Description
 *   Class for XML document.
 */
@interface XmlDoc : XmlObject 
{
	/** document body */
	xmlDocPtr	xml_doc_ptr ;
	/** Input file name. This will be null. */
	char *		input_file_name ;
	/** Output file name. This will be null. */
	char *		output_file_name ;
}

  /**
   * @brief	Initialize the XML processing system.
   * @par Description
   *   Initialize the XML processing system.
   */
  + load ;
  
  /**
   * @brief	Read XML text file and built data structure for it.
   * @return	Generated instance of XmlDoc
   * @param	fname Source file name. If you pass null pointer, 
   *		standard input is used.
   * @param	dovalid Validate the document when parse the file or not.
   * @param	doremov Set true when you want to remove needless spaces 
   *  		from the source XML text.
   * @par Description
   *   Read XML text file and built data structure for it.
   *   If the parsing is succeeded this method returns non nil value.
   */
  + (XmlDoc *) readFile: (const char *) fname doValidate: (BOOL) dovalid
      doRemoveSpace: (BOOL) doremov ;
  
  /**
   * @brief	Make empty XML document.
   * @param	root Root node name. If you pass null pointer,
   *		the default name <code>root</code> is used.
   * @return	Generated instance of XmlDoc
   * @par Description
   *   Read XML text file and built data structure for it.
   */
  + (XmlDoc *) makeEmptyDoc: (const xmlChar *) root ;
  
  /**
   * @brief	Initialize instance of XmlDoc class.
   * @return	Generated instance of XmlDoc
   * @param	doc Source xml data for libxml2.
   * @par Description
   *   Initialize instance of XmlDoc class.
   *   This method is called in the memory allocation function in libxml2.
   *   DO NOT CALL this method directory.
   */
  - initXmlDoc: (xmlDocPtr) doc ;
  
  /**
   * @brief	Get root node.
   * @return	Root node.
   * @par Description
   *   Get root node.
   */
  - (XmlNode *) rootNode ;
  
  /**
   * @brief	Set input file name string.
   * @param	fname Input file name. It will be null.
   * @par Description
   *   Set input file name string.
   *   If the name is null, it means the document does not have
   *   input file or read from standard input.
   */
  - (void) setInputFileName: (const char *) fname ;
  
  /**
   * @brief	Get input file name.
   * @return	Input file name or null.
   * @par Description
   *   Get input file name string.
   *   If the name is null, it means the document does not have
   *   input file or read from standard input.
   */
  - (const char *) inputFileName ;
  
  /**
   * @brief	Set output file name string.
   * @param	fname Output file name. It will be null.
   * @par Description
   *   Set output file name string.
   *   If the name is null, it means the document is saved into
   *   standard output.
   */
  - (void) setOutputFileName: (const char *) fname ;
  
  /**
   * @brief	Save the document into the output file.
   * @return	Number of writen bytes. If some errors were occured,
   *		this value will be negative.
   * @par Description
   *   Save the document into the output file.
   */
  - (int) save ;
  
  /**
   * @brief	Save the document into the output file.
   * @return	Number of writen bytes. If some errors were occured,
   *   		this value will be negative.
   * @param	fname Destination output file name.
   *		If you give null pointer, standard output are used.
   * @par Description
   *   Save the document into the output file.
   */
  - (int) saveFileAs: (const char *) fname ;
  
  /**
   * @brief	Get content of document.
   * @return	xmlDocPtr type data.
   * @par Description
   *   Get content of document.
   */
  - (xmlDocPtr) docPtr ;

@end

#endif /* XMLDOC_H_INCLUDED */

