/**
 * @file	xmllex.h
 * @brief	Lexical analyzer functions to be used by Yacc.
 * @date	2004.02.14
 * @author	Steel Wheels Project
 * @par Copyright
 *   Copyright (C) 2004 Steel Wheels Project.
 *   All right reserved.
 * @par Introduction
 *   This file contains Functions for lexical analysis of XML data.
 *   You can use yacc or bison for syntax check of XML documentation.
 *   The function xml_lex() is designed to replace yylex() generated
 *   by flex.
 * @par Bug?
 *   This function uses YYPURE macro to get re-entrant interface of yylex().
 *   I don't know the macro can be used for the other lex system except flex.
 */

#ifndef XMLLEX_H_INCLUDED
#define XMLLEX_H_INCLUDED

#include <LibXmlKit/xmltoken.h>

@class	XmlDoc ;

/**
 * Macro to generate re-entrant parser.
 */
#define	YYPURE

/**
 * @brief	Reserved word information for XML lexical analyzer.
 * @par Description
 *   Reserved word information for XML lexical analyzer.
 *   "-1" is reserved for id which means the NOT reserved word.
 */
typedef struct {
	/** Id number for the reserved word */
	int			rword_id ;
	/** String of reserved word */
	const xmlChar *		rword_str ;
} XmlRWord ;

/** Pointer of XmlRWord */
typedef XmlRWord *		XmlRWordPtr ;
/** Const pointer of XmlRWord */
typedef const XmlRWord *	XmlConstRWordPtr ;

/**
 * @brief	Main function for lexical analyzer.
 * @return	Type of token defined in yacc header.
 * @param	token   token Destination data to store current token.
 * @par Description
 *   Main function for lexical analyzer.
 *   This function must be compatible with lex's output.
 */
int
xml_lex(XmlTokenPtr token) ;

/**
 * @brief	Setup the environment for lexical analyzer
 * @param 	doc Source XML documentation.
 * @param 	array Array of reserved words for element name.
 * @par Description
 *   Setup the environment for lexical analyzer.
 */
void
xml_lex_setup(XmlDoc * doc, XmlConstRWordPtr array);

/**
 * @brief	Release resources for lexical analyzer.
 * @par Description
 *   Release resources for lexical analyzer. 
 *   You can not call functions for XML processing after calling this method.
 *   If you use the analyzer again, call xml_lex_setup() function.
 */
void
xml_lex_finish(void) ;

/**
 * @brief	Print the error message to error stream.
 * @param	message Source message.
 * @par Description
 *   Print error message to the error stream.
 */
void
xml_lex_error(const char * message) ;

#endif /* XMLLEX_H_INCLUDED */

