/**
 * @file	xmlnode.h
 * @brief	#XmlNode class definition.
 * @date	2004.02.14
 * @author	Steel Wheels Project
 * @par Copyright
 *   Copyright (C) 2004 Steel Wheels Project.
 *   All right reserved.
 * @par Description
 *   #XmlNode class definition.
 *   #XmlNode class operates the node of xml data structure in 
 *   #XmlDoc object.
 */

#ifndef XMLNODE_H_INCLUDED
#define XMLNODE_H_INCLUDED

#include <LibXmlKit/xmlobject.h>

/**
 * States of the xml node.
 */
typedef enum {
	/** Xml node state: initial */
	XML_NODE_STATE_INIT
} XmlNodeState ;

/**
 * @brief	Xml node object.
 * @par Description
 *   Xml node object.
 *   It will be an element, attribute and so on.
 */
@interface XmlNode : XmlObject 
{
	/** Content data of XmlNode class */
	xmlNodePtr	xml_node_ptr ;
	/** State of the node */
	XmlNodeState	node_state ;
}

  /**
   * @brief	Generate new element node.
   * @return	New element node.
   * @param	name Name of element node.
   * @par Description
   *   Generate new element node.
   */
  + newElement: (const xmlChar *) name ;

  /**
   * @brief	Generate new text node.
   * @return	New text node.
   * @param	text Content of new text node.
   * @par Description
   *    Generate new text node.
   */
  + newText: (const xmlChar *) text ;
  
  /**
   * @brief	Initialize the instance of XmlNode class.
   * @return	Generated instance of XmlNode
   * @param	node Source xml node.
   * @par Description
   *   Initialize the instance of XmlNode class.
   *   This method is called by the memory allocation function in libxml2.
   *   DO NOT CALL this method directory.
   */
  - initXmlNode: (xmlNodePtr) node ;
  
  /**
   * @brief	Get content data.
   * @return	xmlNodePtr data.
   * @par Description
   *   Get content data.
   *   This method is used for XmlDoc class.
   *   DO NOT CALL this in the other places.
   */
  - (xmlNodePtr) nodePtr ;
  
  /**
   * @brief	Get type of source node.
   * @return	Type of source node.
   * @par Description
   *   Get type of source node.
   */
  - (xmlElementType) type ;
  
  /**
   * @brief	 Get state of the node.
   * @return	State of the node.
   * @par Description
   *   Get state of the node.
   */
  - (XmlNodeState) state ;
  
  /**
   * @brief	Get name of element node.
   * @return	Name of the element node.
   *   		If source node is not element node, the result will be null,
   * @par Description
   *   Get name of element node.
   *   If source node is not element node, the result will be null,
   */
  - (const xmlChar *) name ;
  
  /**
   * @brief	Set name of element node.
   * @param	name
   * @par Description
   *   Set name of element node.
   */
  - (void) setName: (const xmlChar *) name ;
 
  /**
   * @brief	Set property to the element node.
   * @param	name Property name.
   * @param	value Property value. 
   *		If you set null. the attribute is removed.
   * @par Description
   *   Set property to the element node.
   *   If same name property is already exist, the value is overwritten.
   *   If value is null, the attribute is removed.
   */
  - setAttr: (const xmlChar *) name value: (const xmlChar *) value ;

  /**
   * @brief	Get attribute value from element node.
   * @param	name Destination attribute name.
   * @par Description
   *   Get attribute value from element node.
   */
  - (const xmlChar *) getAttrValue: (const xmlChar *) name ;

  /**
   * @brief	Get line number which contains the node.
   * @return	If the value is found, return the positive value.
   *		But it is not found, returns negative value.
   * @par Description
   *   Get line number of the input file which contains the node.
   */
  - (int) lineNo ;
  
  /**
   * @brief	Set text to text node.
   * @param	src Source text.
   * @param	len Length of the source text.
   * @par Description
   *   Overwrite text for text node.
   *   Current context will be lose.
   */
  - (void) setText: (const xmlChar *) src length: (size_t) len ;

  /**
   * @brief	Get text from text node.
   * @return	Content text.
   * @par Description
   *   Get content of text node.
   */
  - (const xmlChar *) getText ;

  /**
   * @brief	Return true if node is empty text node.
   * @return	True if the node is empty text node.
   * @par Description
   *    Return true when the node is text node and contains only spaces.
   *    If the parameter is not text node, this method returns false.
   */
  - (BOOL) isBlankText ;
  
  /**
   * @brief	Get next sibling.
   * @return	Next sibling node or nil (when it is not found).
   * @par Description
   *   Get next sibling. If it is not found, nil is returned.
   */
  - (XmlNode *) next ;
  
  /**
   * @brief	Get next sibling which matches given type.
   * @return	Next sibling node or nil (when it is not found).
   * @param	type Type to search.
   * @par Description
   *   Get next sibling which has given type.
   *   If it is not found, nil is returned.
   */
  - (XmlNode *) next: (xmlElementType) type ;
  
  /**
   * @brief	Get next sibling which matches given types.
   * @return	Next sibling node or nil (when it is not found).
   * @param	typ1 Type to search.
   * @param	typ2 Other type to search.
   * @par Description
   *   Get next sibling which has given types.
   *   If it is not found, nil is returned.
   */
  - (XmlNode *) next: (xmlElementType) typ1 or: (xmlElementType) typ2 ;
  
  /**
   * @brief	Get parent node.
   * @return	Parent node or nil (when it is root node).
   * @par Description
   *   Get parent node. If it is not found, nil is returned.
   */
  - (XmlNode *) parent ;
  
  /**
   * @brief	Get 1st child node.
   * @return	1st child or nil if it is not found.
   * @par Description
   *   Get 1st child node.
   *   If it is not found, nil is returned.
   */
  - (XmlNode *) child ;
  
  /**
   * @brief	Get child which has given type.
   * @return	Child node or nil (when it is not found).
   * @param	type Type of node to search.
   * @par Description
   *   Get child which has given type.
   *   If it is not found, nil is returned.
   * @warning
   *   The blank text node is ignored in this methid.
   */
  - (XmlNode *) child: (xmlElementType) type ;
  
  /**
   * @brief	Get child node which matches given types.
   * @return	Child node or nil (when it is not found).
   * @param	typ1 Type to search.
   * @param	typ2 Other type to search.
   * @par Description
   *   Get child node which has given types.
   *   If it is not found, nil is returned.
   * @warning
   *   The blank text node is ignored in this methid.
   */
  - (XmlNode *) child: (xmlElementType) typ1 or: (xmlElementType) typ2 ;
  
  /**
   * @brief	Get last child node.
   * @return	Last child or nil if it is not found.
   * @par Description
   *   Get last child node.
   *   If it is not found, nil is returned.
   */
  - (XmlNode *) lastChild ;
  
  /**
   * @brief	Get last child which has given type.
   * @return	Child node or nil (when it is not found).
   * @param	type Type of node to search.
   * @par Description
   *   Get last child which has given type.
   *   If it is not found, nil is returned.
   * @warning
   *   The blank text node is ignored in this methid.
   */
  - (XmlNode *) lastChild: (xmlElementType) type ;
  
  /**
   * @brief	Get last child node which matches given types.
   * @return	Child node or nil (when it is not found).
   * @param	typ1 Type to search.
   * @param	typ2 Other type to search.
   * @par Description
   *   Get last child node which has given types.
   *   If it is not found, nil is returned.
   * @warning
   *   The blank text node is ignored in this methid.
   */
  - (XmlNode *) lastChild: (xmlElementType) typ1 or: (xmlElementType) typ2 ;
  
  /**
   * @brief	Add node as new sibling.
   * @param	node Source node list to add.
   * @param	followers If this is true, 
   *            followers are also added as children.
   * @par Description	
   *   Add node (and it's followers) as new sibling.
   *   Before adding node, the node is duplicated.
   */
  - addSibling: (XmlNode *) node withFollowers: (BOOL) followers ;

  /**
   * @brief	Add node as new children.
   * @param	node Source node list to add.
   * @param	followers If this is true, 
   *            followers are also added as children.
   * @par Description	
   *   Add node (and it's followers) as new children.
   *   Before adding node, the node is duplicated.
   */
  - addChild: (XmlNode *) node withFollowers: (BOOL) followers ;
  
  /**
   * @brief	Duplicate the node.
   * @param	withchild Set true the duplication of children of the node
   *		is required.
   * @par Description
   *   Duplicate the node. If "withchild" is true, all children also copied.
   */
  - duplicate: (BOOL) withchild ;
  
  /**
   * @brief	Unlink the connection of the node.
   * @param	withfollowers Set true if you want to unlink all followers
   * 		of the node.
   * @par Description
   *    Unlink the connection of the node.
   */
  - (void) unlink: (BOOL) withfollowers ;
 
  /**
   * @brief	Replace this node by source node.
   * @param	src Source node.
   * @par Description
   *   Replace this node by source node.
   * @warning
   *   This node is unlinked after this operation.
   */
  - (void) replaceBy: (XmlNode *) src ;

  /**
   * @brief	Print the content of node.
   * @param	outfp Output file stream.
   * @return	Return nil if execution is failed.
   * @par Description
   *  Print the content of node.
   *  If some error occured, this method returns nil.
   */
  - (void) print: (FILE *) outfp ;

@end

#endif /* XMLNODE_H_INCLUDED */

