/**
 * @file	xmltoken.h
 * @brief	#XmlToken data structure and functions for it.
 * @date	2004.02.14
 * @author	Steel Wheels Project
 * @par Copyright
 *   Copyright (C) 2004 Steel Wheels Project.
 *   All right reserved.
 * @par Description
 *   #XmlToken data structure and functions for it.
 *   The data structure is used by lex and yacc program as data type of token.
 */

#ifndef XMLTOKEN_H_INCLUDED
#define XMLTOKEN_H_INCLUDED

#include <LibXmlKit/xmlnode.h>

/**
 * The type of current XML token for lexical analyzer.
 */
typedef enum {
	XML_TOKEN_STATE_ELM_HDR,
	XML_TOKEN_STATE_ELM_NAME,
	XML_TOKEN_STATE_ELM_END,
	XML_TOKEN_STATE_TEXT,
	XML_TOKEN_STATE_FINISH
} XmlTokenState ;

/** Constant characters for XML lexical analyzer and syntax parser */
static const int XML_TOKEN_CHAR_ELM_BEGIN	= '<' ;
static const int XML_TOKEN_CHAR_ELM_END		= '>' ;
static const int XML_TOKEN_CHAR_TEXT		= '#' ;
static const int XML_TOKEN_CHAR_EOF		= EOF ;

/**
 * @brief	XML token data.
 * @par Description	
 *    Data type for XML token data.
 *    This data is used as token in lexical analyzer and syntax parser (yacc).
 */
typedef struct {
	/** State of XML token */
	XmlTokenState		token_state ;
	/** XML node */
	XmlNode *		token_node ;
} XmlToken ;

/**
 * @brief	Pointer of XmlToken structure.
 * @par Description
 *    Pointer of XmlToken structure.
 */
typedef XmlToken *		XmlTokenPtr ;

/**
 * @brief	Pointer of constant XmlToken structure.
 * @par Description
 *    Pointer of constant XmlToken structure.
 */
typedef const XmlToken *	XmlConstTokenPtr ;

/**
 * @brief	Set content of XML token.
 * @param	dst Destination XML token.
 * @param	state State of token.
 * @param	node Content of token.
 * @par Description
 *   Set content of XML token.
 */
static inline void
xml_token_set(XmlTokenPtr dst, XmlTokenState state, XmlNode * node)
{
	dst->token_state = state ;
	dst->token_node = node ;
}

/**
 * @brief	Print the content of token.
 * @param	outfp Output file stream.
 * @param	token Source token to print.
 * @par Description
 *    Print the content of token.
 */
void
xml_token_print(FILE * outfp, XmlConstTokenPtr token) ;

#endif /* XMLTOKEN_H_INCLUDED */

